/**
 * initApp handles setting up UI event listeners and registering Firebase auth listeners:
 *  - firebase.auth().onAuthStateChanged: This listener is called when the user is signed in or
 *    out, and that is where we update the UI.
 */

let email;
let uid;
let restName;
let restId;
let traderName;
let traderId;
let branchName;
let branchId;
let drinks = false;
let sideMeal = false;
let condiments = false;
let extras = false;
let note = " ";
let calories = " ";
let restPhoto = " ";
let mealDiscount = 0;
let drinksArray = [];
let drinksArrayIds = [];
let sideMealArray = [];
let sideMealIds = [];
let condimentsArray = [];
let condimentsArrayIds = [];
let extrasArray = [];
let extrasArrayIds = [];
let db = firebase.firestore();
let username;

var bs_modal = $("#modal");
var image = document.getElementById("image");
var cropper, reader, file;
let downloadUrl = " ";

var maxwidth = 512;
var maxheight = 512;

var finalAspectRatio = 1;

var imgwidth = 0;
var imgheight = 0;
let mealQuant = "0";
let mealsArray = [];

$("body").on("change", ".image", function (e) {
  var files = e.target.files;
  var done = function (url) {
    image.src = url;

    image.onload = function () {
      imgwidth = this.width;
      imgheight = this.height;
      if (imgwidth === maxwidth && imgheight === maxheight) {
        console.log("Image size is " + maxwidth + "X" + maxheight);
        bs_modal.modal("show");
      } else {
        Swal.fire({
          icon: "error",
          title: "Oops...",
          text: "Image size must be " + maxwidth + "w by " + maxheight + "h",
        });
      }
    };
  };

  if (files && files.length > 0) {
    file = files[0];

    if (URL) {
      done(URL.createObjectURL(file));
    } else if (FileReader) {
      reader = new FileReader();
      reader.onload = function (e) {
        done(reader.result);
      };
      reader.readAsDataURL(file);
    }
  }
});

bs_modal
  .on("shown.bs.modal", function () {
    $("#modal").modal({
      backdrop: "static",
      keyboard: false,
    });
    cropper = new Cropper(image, {
      aspectRatio: finalAspectRatio,
      viewMode: 1,
      preview: ".preview",
    });
  })
  .on("hidden.bs.modal", function () {
    cropper.destroy();
    cropper = null;
  });

$("#crop").click(function () {
  document.getElementById("crop").disabled = true;

  canvas = cropper.getCroppedCanvas({
    imageSmoothingEnabled: true,
    imageSmoothingQuality: "high",
  });

  canvas.toBlob(function (blob) {
    url = URL.createObjectURL(blob);
    let imPreview = document.getElementById("imgPreview");

    var reader = new FileReader();
    reader.readAsDataURL(blob);
    reader.onloadend = function () {
      var base64data = reader.result;
      console.log(base64data);
      // Get a reference to the storage service, which is used to create references in your storage bucket
      const storage = firebase.storage();

      // Create a storage reference from our storage service
      const storageRef = storage.ref();
      storageRef
        .child("meals/" + generateUUID())
        .putString(base64data, "data_url")
        .then((snapshot) => {
          snapshot.ref
            .getDownloadURL()
            .then(function (downloadURL) {
              console.log("File available at", downloadURL);
              downloadUrl = downloadURL;
              imPreview.src = downloadURL;
              document.getElementById("crop").disabled = false;

              bs_modal.modal("hide");
            })
            .catch(function (error) {
              document.getElementById("crop").disabled = false;

              console.log("Error getting document:", error);
            });
          console.log("Uploaded a data_url string!");
        });
    };
  });
});

function initApp() {
  // Listening for auth state changes.
  // [START authstatelistener]
  firebase.auth().onAuthStateChanged(function (user) {
    if (user) {
      // User is signed in.

      email = user.email;
      let emailVerified = user.emailVerified;
      uid = user.uid;
      let providerData = user.providerData;
      username = user.displayName;

      // console.log(email)
      let db = firebase.firestore();

      let docRef = db.collection("branches").doc(uid);

      docRef
        .get()
        .then(function (doc) {
          if (doc.exists) {
            restName = doc.data().restaurant;
            restId = doc.data().restId;
            traderId = doc.data().branchId;
            traderName = doc.data().managerFullName;
            branchName = doc.data().branch;

            console.log("rest Id: " + restId);
            document.getElementById("proName").innerText =
              traderName + " (" + branchName + ")";

            var divDrinks = document.getElementById("chooseDrinks");
            var divCondiments = document.getElementById("chooseCondiments");
            var divSides = document.getElementById("chooseSide");
            var divExtras = document.getElementById("chooseExtras");

            db.collection("meals")
              .where("restId", "==", restId)
              .orderBy("timestamp", "desc")
              .onSnapshot((snapshot) => {
                snapshot.forEach(function (docs) {
                  let mealId = docs.id;
                  let mealImg = docs.data().mealImage;
                  let mealName = docs.data().mealName;
                  let mealDesc = docs.data().mealDesc;
                  let mealPrice = docs.data().mealPrice;
                  let mealCategory = docs.data().restCategories;
                  let mealCategoryId = docs.data().restCategoriesIds;
                  //console.log("meals: "+ JSON.stringify(docs.data()));
                  let newObject = docs.data();
                  newObject.id = docs.id;
                  mealsArray.push(newObject);

                  if (
                    mealCategory.includes("Non-Alcoholic Drink") ||
                    mealCategory.includes("Alcohol") ||
                    mealCategory.includes("Juice & Smoothies")
                  ) {
                    //In the array!
                    divDrinks.innerHTML +=
                      "<option value=" + mealId + ">" + mealName + "</option>";
                  }
                  if (mealCategory.includes("Condiment")) {
                    //In the array!
                    divCondiments.innerHTML +=
                      "<option value=" + mealId + ">" + mealName + "</option>";
                  }
                  divExtras.innerHTML +=
                    "<option value=" +
                    mealId +
                    "><img src=" +
                    mealImg +
                    'width="20" height="15">' +
                    mealName +
                    "</option>";
                  divSides.innerHTML +=
                    "<option value=" + mealId + ">" + mealName + "</option>";
                });
              });
          }
        })
        .catch(function (error) {
          console.log("Error getting document:", error);
        });
    } else {
      // User is signed out.
      console.log("User is signed out");
      window.location.href = "index.html";
    }
  });
  // [END authstatelistener]
}

window.onload = function () {
  initApp();
};

function toggleSignOut() {
  // [START signout]
  firebase.auth().signOut();
  // [END signout]
}

$(document).on("keydown", function (e) {
  var keyCode = e.which || e.keyCode;
  if (keyCode == 13) {
    // enter key code
    toggleSignIn();
  }
});
$(document).ready(function () {
  $("#mealCategory").select2();
  $("#chooseDrinks").select2();
  $("#chooseCondiments").select2();
  $("#chooseSide").select2();
  $("#chooseExtras").select2();

  $("#modal").modal({
    backdrop: "static",
    keyboard: false,
  });

  $("#chooseDrinks").on("select2:select", function (e) {
    var data = e.params.data;

    let mealData = mealsArray.filter((e) => e.id === data.id)[0];

    console.log("Selected Meal: " + JSON.stringify(mealData));
    var amount = "di" + data.id;
    var amountLabel = "dlabel" + data.id;
    var isPaid = "dcheckbox" + data.id;
    var isPaidLabel = "dlabel2" + data.id;
    var isOptional = "dcheckbox2" + data.id;
    var isOptionalLabel = "dlabel3" + data.id;
    const i = document.createElement("input");
    const label = document.createElement("label");
    label.htmlFor = amount;
    label.id = amountLabel;
    label.appendChild(
      document.createTextNode("Options for " + data.text + ": ")
    );

    i.setAttribute("type", "text");
    i.setAttribute("id", amount);
    i.setAttribute("class", "fw-700 fs-16 form-control");
    i.setAttribute("placeholder", "Amount is disabled.");
    i.setAttribute("name", "username");
    i.disabled = true;

    var elem = document.createElement("img");
    if (mealData["mealImage"] != " ") {
      elem.src = mealData["mealImage"];
    } else {
      elem.src = "../../images/noimage.png";
    }

    elem.setAttribute("height", "200");
    elem.setAttribute("width", "50");
    elem.setAttribute("alt", "Flower");
    elem.setAttribute("id", "dimg" + data.id);

    const checkbox = document.createElement("input");
    checkbox.setAttribute("type", "checkbox");
    checkbox.setAttribute("id", isPaid);
    checkbox.setAttribute("value", true);
    const label2 = document.createElement("label");
    label2.htmlFor = isPaid;
    label2.id = isPaidLabel;
    label2.appendChild(document.createTextNode("Must be paid for"));

    const checkbox2 = document.createElement("input");
    checkbox2.setAttribute("type", "checkbox");
    checkbox2.setAttribute("id", isOptional);
    checkbox2.setAttribute("value", true);
    const label3 = document.createElement("label");
    label3.htmlFor = isOptional;
    label3.id = isOptionalLabel;
    label3.appendChild(document.createTextNode("Is Optional"));

    const f = document.getElementById("drinksForm");
    f.appendChild(elem);
    f.appendChild(label);
    f.appendChild(i);
    f.appendChild(checkbox);
    f.appendChild(label2);
    f.appendChild(checkbox2);
    f.appendChild(label3);

    checkbox.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox is checked..");
        i.disabled = false;
        i.setAttribute("value", "0");
        i.setAttribute("placeholder", "Enter amount");
      } else {
        console.log("Checkbox is not checked..");
        i.disabled = true;

        i.setAttribute("placeholder", "Amount is disabled.");
      }
    });

    checkbox2.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox2 is checked..");
      } else {
        console.log("Checkbox2 is not checked..");
      }
    });
  });

  $("#chooseDrinks").on("select2:unselect", function (e) {
    var data = e.params.data;
    console.log(data);

    const f = document.getElementById("drinksForm");
    var amount = "di" + data.id;
    var amountLabel = "dlabel" + data.id;
    var isPaid = "dcheckbox" + data.id;
    var isPaidLabel = "dlabel2" + data.id;
    var isOptional = "dcheckbox2" + data.id;
    var isOptionalLabel = "dlabel3" + data.id;
    var extraImg = "dimg" + data.id;

    f.removeChild(document.getElementById(amount));
    f.removeChild(document.getElementById(amountLabel));
    f.removeChild(document.getElementById(isPaidLabel));
    f.removeChild(document.getElementById(isPaid));
    f.removeChild(document.getElementById(isOptionalLabel));
    f.removeChild(document.getElementById(isOptional));
    f.removeChild(document.getElementById(extraImg));
  });
  $("#chooseCondiments").on("select2:select", function (e) {
    var data = e.params.data;

    let mealData = mealsArray.filter((e) => e.id === data.id)[0];

    console.log("Selected Meal: " + JSON.stringify(mealData));
    var amount = "ci" + data.id;
    var amountLabel = "clabel" + data.id;
    var isPaid = "ccheckbox" + data.id;
    var isPaidLabel = "clabel2" + data.id;
    var isOptional = "ccheckbox2" + data.id;
    var isOptionalLabel = "clabel3" + data.id;
    const i = document.createElement("input");
    const label = document.createElement("label");
    label.htmlFor = amount;
    label.id = amountLabel;
    label.appendChild(
      document.createTextNode("Options for " + data.text + ": ")
    );

    i.setAttribute("type", "text");
    i.setAttribute("id", amount);
    i.setAttribute("class", "fw-700 fs-16 form-control");
    i.setAttribute("placeholder", "Amount is disabled.");
    i.setAttribute("name", "username");
    i.disabled = true;

    var elem = document.createElement("img");
    if (mealData["mealImage"] != " ") {
      elem.src = mealData["mealImage"];
    } else {
      elem.src = "../../images/noimage.png";
    }

    elem.setAttribute("height", "200");
    elem.setAttribute("width", "50");
    elem.setAttribute("alt", "Flower");
    elem.setAttribute("id", "cimg" + data.id);

    const checkbox = document.createElement("input");
    checkbox.setAttribute("type", "checkbox");
    checkbox.setAttribute("id", isPaid);
    checkbox.setAttribute("value", true);
    const label2 = document.createElement("label");
    label2.htmlFor = isPaid;
    label2.id = isPaidLabel;
    label2.appendChild(document.createTextNode("Must be paid for"));

    const checkbox2 = document.createElement("input");
    checkbox2.setAttribute("type", "checkbox");
    checkbox2.setAttribute("id", isOptional);
    checkbox2.setAttribute("value", true);
    const label3 = document.createElement("label");
    label3.htmlFor = isOptional;
    label3.id = isOptionalLabel;
    label3.appendChild(document.createTextNode("Is Optional"));

    const f = document.getElementById("condimentsForm");
    f.appendChild(elem);
    f.appendChild(label);
    f.appendChild(i);
    f.appendChild(checkbox);
    f.appendChild(label2);
    f.appendChild(checkbox2);
    f.appendChild(label3);

    checkbox.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox is checked..");
        i.disabled = false;
        i.setAttribute("value", "0");
        i.setAttribute("placeholder", "Enter amount");
      } else {
        console.log("Checkbox is not checked..");
        i.disabled = true;

        i.setAttribute("placeholder", "Amount is disabled.");
      }
    });

    checkbox2.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox2 is checked..");
      } else {
        console.log("Checkbox2 is not checked..");
      }
    });
  });

  $("#chooseCondiments").on("select2:unselect", function (e) {
    var data = e.params.data;
    console.log(data);

    const f = document.getElementById("condimentsForm");
    var amount = "ci" + data.id;
    var amountLabel = "clabel" + data.id;
    var isPaid = "ccheckbox" + data.id;
    var isPaidLabel = "clabel2" + data.id;
    var isOptional = "ccheckbox2" + data.id;
    var isOptionalLabel = "clabel3" + data.id;
    var extraImg = "cimg" + data.id;

    f.removeChild(document.getElementById(amount));
    f.removeChild(document.getElementById(amountLabel));
    f.removeChild(document.getElementById(isPaidLabel));
    f.removeChild(document.getElementById(isPaid));
    f.removeChild(document.getElementById(isOptionalLabel));
    f.removeChild(document.getElementById(isOptional));
    f.removeChild(document.getElementById(extraImg));
  });
  $("#chooseSide").on("select2:select", function (e) {
    var data = e.params.data;

    let mealData = mealsArray.filter((e) => e.id === data.id)[0];

    console.log("Selected Meal: " + JSON.stringify(mealData));
    var amount = "si" + data.id;
    var amountLabel = "slabel" + data.id;
    var isPaid = "scheckbox" + data.id;
    var isPaidLabel = "slabel2" + data.id;
    var isOptional = "scheckbox2" + data.id;
    var isOptionalLabel = "slabel3" + data.id;
    const i = document.createElement("input");
    const label = document.createElement("label");
    label.htmlFor = amount;
    label.id = amountLabel;
    label.appendChild(
      document.createTextNode("Options for " + data.text + ": ")
    );

    i.setAttribute("type", "text");
    i.setAttribute("id", amount);
    i.setAttribute("class", "fw-700 fs-16 form-control");
    i.setAttribute("placeholder", "Amount is disabled.");
    i.setAttribute("name", "username");
    i.disabled = true;

    var elem = document.createElement("img");
    if (mealData["mealImage"] != " ") {
      elem.src = mealData["mealImage"];
    } else {
      elem.src = "../../images/noimage.png";
    }

    elem.setAttribute("height", "200");
    elem.setAttribute("width", "50");
    elem.setAttribute("alt", "Flower");
    elem.setAttribute("id", "simg" + data.id);

    const checkbox = document.createElement("input");
    checkbox.setAttribute("type", "checkbox");
    checkbox.setAttribute("id", isPaid);
    checkbox.setAttribute("value", true);
    const label2 = document.createElement("label");
    label2.htmlFor = isPaid;
    label2.id = isPaidLabel;
    label2.appendChild(document.createTextNode("Must be paid for"));

    const checkbox2 = document.createElement("input");
    checkbox2.setAttribute("type", "checkbox");
    checkbox2.setAttribute("id", isOptional);
    checkbox2.setAttribute("value", true);
    const label3 = document.createElement("label");
    label3.htmlFor = isOptional;
    label3.id = isOptionalLabel;
    label3.appendChild(document.createTextNode("Is Optional"));

    const f = document.getElementById("sideForm");

    f.appendChild(elem);
    f.appendChild(label);
    f.appendChild(i);
    f.appendChild(checkbox);
    f.appendChild(label2);
    f.appendChild(checkbox2);
    f.appendChild(label3);

    checkbox.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox is checked..");
        i.disabled = false;
        i.setAttribute("value", "0");
        i.setAttribute("placeholder", "Enter amount");
      } else {
        console.log("Checkbox is not checked..");
        i.disabled = true;

        i.setAttribute("placeholder", "Amount is disabled.");
      }
    });

    checkbox2.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox2 is checked..");
      } else {
        console.log("Checkbox2 is not checked..");
      }
    });
  });

  $("#chooseSide").on("select2:unselect", function (e) {
    var data = e.params.data;
    console.log(data);

    const f = document.getElementById("sideForm");
    var amount = "si" + data.id;
    var amountLabel = "slabel" + data.id;
    var isPaid = "scheckbox" + data.id;
    var isPaidLabel = "slabel2" + data.id;
    var isOptional = "scheckbox2" + data.id;
    var isOptionalLabel = "slabel3" + data.id;
    var extraImg = "simg" + data.id;

    f.removeChild(document.getElementById(amount));
    f.removeChild(document.getElementById(amountLabel));
    f.removeChild(document.getElementById(isPaidLabel));
    f.removeChild(document.getElementById(isPaid));
    f.removeChild(document.getElementById(isOptionalLabel));
    f.removeChild(document.getElementById(isOptional));
    f.removeChild(document.getElementById(extraImg));
  });

  $("#chooseExtras").on("select2:select", function (e) {
    var data = e.params.data;

    let mealData = mealsArray.filter((e) => e.id === data.id)[0];

    console.log("Selected Meal: " + JSON.stringify(mealData));
    var amount = "ei" + data.id;
    var amountLabel = "elabel" + data.id;
    var isPaid = "echeckbox" + data.id;
    var isPaidLabel = "elabel2" + data.id;
    var isOptional = "echeckbox2" + data.id;
    var isOptionalLabel = "elabel3" + data.id;
    const i = document.createElement("input");
    const label = document.createElement("label");
    label.htmlFor = amount;
    label.id = amountLabel;
    label.appendChild(
      document.createTextNode("Options for " + data.text + ": ")
    );

    i.setAttribute("type", "text");
    i.setAttribute("id", amount);
    i.setAttribute("class", "fw-700 fs-16 form-control");
    i.setAttribute("placeholder", "Amount is disabled.");
    i.setAttribute("name", "username");
    i.disabled = true;

    var elem = document.createElement("img");
    if (mealData["mealImage"] != " ") {
      elem.src = mealData["mealImage"];
    } else {
      elem.src = "../../images/noimage.png";
    }

    elem.setAttribute("height", "200");
    elem.setAttribute("width", "50");
    elem.setAttribute("alt", "Flower");
    elem.setAttribute("id", "eimg" + data.id);

    const checkbox = document.createElement("input");
    checkbox.setAttribute("type", "checkbox");
    checkbox.setAttribute("id", isPaid);
    checkbox.setAttribute("value", true);
    const label2 = document.createElement("label");
    label2.htmlFor = isPaid;
    label2.id = isPaidLabel;
    label2.appendChild(document.createTextNode("Must be paid for"));

    const checkbox2 = document.createElement("input");
    checkbox2.setAttribute("type", "checkbox");
    checkbox2.setAttribute("id", isOptional);
    checkbox2.setAttribute("value", true);
    const label3 = document.createElement("label");
    label3.htmlFor = isOptional;
    label3.id = isOptionalLabel;
    label3.appendChild(document.createTextNode("Is Optional"));

    const f = document.getElementById("extrasForm");
    f.appendChild(elem);
    f.appendChild(label);
    f.appendChild(i);
    f.appendChild(checkbox);
    f.appendChild(label2);
    f.appendChild(checkbox2);
    f.appendChild(label3);

    checkbox.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox is checked..");
        i.disabled = false;
        i.setAttribute("value", "0");
        i.setAttribute("placeholder", "Enter amount");
      } else {
        console.log("Checkbox is not checked..");
        i.disabled = true;

        i.setAttribute("placeholder", "Amount is disabled.");
      }
    });

    checkbox2.addEventListener("change", function () {
      if (this.checked) {
        console.log("Checkbox2 is checked..");
      } else {
        console.log("Checkbox2 is not checked..");
      }
    });
  });

  $("#chooseExtras").on("select2:unselect", function (e) {
    var data = e.params.data;
    console.log(data);

    const f = document.getElementById("extrasForm");
    var amount = "ei" + data.id;
    var amountLabel = "elabel" + data.id;
    var isPaid = "echeckbox" + data.id;
    var isPaidLabel = "elabel2" + data.id;
    var isOptional = "echeckbox2" + data.id;
    var isOptionalLabel = "elabel3" + data.id;
    var extraImg = "eimg" + data.id;

    f.removeChild(document.getElementById(amount));
    f.removeChild(document.getElementById(amountLabel));
    f.removeChild(document.getElementById(isPaidLabel));
    f.removeChild(document.getElementById(isPaid));
    f.removeChild(document.getElementById(isOptionalLabel));
    f.removeChild(document.getElementById(isOptional));
    f.removeChild(document.getElementById(extraImg));
  });
});
let extrasOptions = [];
let drinksOptions = [];
let condimentsOptions = [];
let sideMealOptions = [];
function saveDrinks() {
  drinksOptions = [];
  if ($("#chooseDrinks").select2("val").length >= 1) {
    let extrasFormIds = $("#chooseDrinks").select2("val");
    let stFromExtras = $("#chooseDrinks").select2("data");
    for (let j = 0; j < stFromExtras.length; j++) {
      let mealData = mealsArray.filter((e) => e.id === extrasFormIds[j])[0];

      let isPaid = document.getElementById(
        "dcheckbox" + extrasFormIds[j]
      ).checked;
      let isOptional = document.getElementById(
        "dcheckbox2" + extrasFormIds[j]
      ).checked;

      if (isPaid) {
        amount = document.getElementById("di" + extrasFormIds[j]).value;
      } else {
        amount = 0;
      }
      console.log(document.getElementById("di" + extrasFormIds[j]).value);
      mealData.extraId = extrasFormIds[j];

      (mealData.extra = stFromExtras[j].text),
        (mealData.amount = amount),
        (mealData.isPaid = isPaid),
        (mealData.isOptional = isOptional),
        drinksOptions.push(mealData);
    }
    var result = document.getElementById("drinksView");
    result.innerHTML = "";
    for (var i = 0; i < drinksOptions.length; i++) {
      result.innerHTML +=
        "Option " +
        (i + 1) +
        ":" +
        "<br>" +
        drinksOptions[i]["extra"] +
        "<br> Amount: " +
        drinksOptions[i]["amount"] +
        "<br> Is Paid: " +
        drinksOptions[i]["isPaid"] +
        "<br> Is Optional: " +
        drinksOptions[i]["isOptional"] +
        "<br><br>";
    }
    $("#modal-drinks").modal("hide");
  }
}
function saveCondiments() {
  condimentsOptions = [];
  if ($("#chooseCondiments").select2("val").length >= 1) {
    let extrasFormIds = $("#chooseCondiments").select2("val");
    let stFromExtras = $("#chooseCondiments").select2("data");
    for (let j = 0; j < stFromExtras.length; j++) {
      let mealData = mealsArray.filter((e) => e.id === extrasFormIds[j])[0];

      let isPaid = document.getElementById(
        "ccheckbox" + extrasFormIds[j]
      ).checked;
      let isOptional = document.getElementById(
        "ccheckbox2" + extrasFormIds[j]
      ).checked;

      if (isPaid) {
        amount = document.getElementById("ci" + extrasFormIds[j]).value;
      } else {
        amount = 0;
      }
      console.log(document.getElementById("ci" + extrasFormIds[j]).value);
      mealData.extraId = extrasFormIds[j];

      (mealData.extra = stFromExtras[j].text),
        (mealData.amount = amount),
        (mealData.isPaid = isPaid),
        (mealData.isOptional = isOptional),
        condimentsOptions.push(mealData);
    }
    var result = document.getElementById("condimentsView");
    result.innerHTML = "";
    for (var i = 0; i < condimentsOptions.length; i++) {
      result.innerHTML +=
        "Option " +
        (i + 1) +
        ":" +
        "<br>" +
        condimentsOptions[i]["extra"] +
        "<br> Amount: " +
        condimentsOptions[i]["amount"] +
        "<br> Is Paid: " +
        condimentsOptions[i]["isPaid"] +
        "<br> Is Optional: " +
        condimentsOptions[i]["isOptional"] +
        "<br><br>";
    }
    $("#modal-condiments").modal("hide");
  }
}
function saveSide() {
  sideMealOptions = [];
  if ($("#chooseSide").select2("val").length >= 1) {
    let extrasFormIds = $("#chooseSide").select2("val");
    let stFromExtras = $("#chooseSide").select2("data");
    for (let j = 0; j < stFromExtras.length; j++) {
      let mealData = mealsArray.filter((e) => e.id === extrasFormIds[j])[0];

      let isPaid = document.getElementById(
        "scheckbox" + extrasFormIds[j]
      ).checked;
      let isOptional = document.getElementById(
        "scheckbox2" + extrasFormIds[j]
      ).checked;

      if (isPaid) {
        amount = document.getElementById("si" + extrasFormIds[j]).value;
      } else {
        amount = 0;
      }
      console.log(document.getElementById("si" + extrasFormIds[j]).value);
      mealData.extraId = extrasFormIds[j];

      (mealData.extra = stFromExtras[j].text),
        (mealData.amount = amount),
        (mealData.isPaid = isPaid),
        (mealData.isOptional = isOptional),
        sideMealOptions.push(mealData);
    }
    var result = document.getElementById("sideView");
    result.innerHTML = "";
    for (var i = 0; i < sideMealOptions.length; i++) {
      result.innerHTML +=
        "Option " +
        (i + 1) +
        ":" +
        "<br>" +
        sideMealOptions[i]["extra"] +
        "<br> Amount: " +
        sideMealOptions[i]["amount"] +
        "<br> Is Paid: " +
        sideMealOptions[i]["isPaid"] +
        "<br> Is Optional: " +
        sideMealOptions[i]["isOptional"] +
        "<br><br>";
    }
    $("#modal-side-meal").modal("hide");
  }
}
function saveExtras() {
  extrasOptions = [];
  if ($("#chooseExtras").select2("val").length >= 1) {
    let extrasFormIds = $("#chooseExtras").select2("val");
    let stFromExtras = $("#chooseExtras").select2("data");
    for (let j = 0; j < stFromExtras.length; j++) {
      let mealData = mealsArray.filter((e) => e.id === extrasFormIds[j])[0];

      let isPaid = document.getElementById(
        "echeckbox" + extrasFormIds[j]
      ).checked;
      let isOptional = document.getElementById(
        "echeckbox2" + extrasFormIds[j]
      ).checked;

      if (isPaid) {
        amount = document.getElementById("ei" + extrasFormIds[j]).value;
      } else {
        amount = 0;
      }
      console.log(document.getElementById("ei" + extrasFormIds[j]).value);
      mealData.extraId = extrasFormIds[j];

      (mealData.extra = stFromExtras[j].text),
        (mealData.amount = amount),
        (mealData.isPaid = isPaid),
        (mealData.isOptional = isOptional),
        extrasOptions.push(mealData);
    }
    console.log("Extras: " + JSON.stringify(extrasOptions));
    var result = document.getElementById("extrasView");
    result.innerHTML = "";
    for (var i = 0; i < extrasOptions.length; i++) {
      result.innerHTML +=
        "Option " +
        (i + 1) +
        ":" +
        "<br>" +
        extrasOptions[i]["extra"] +
        "<br> Amount: " +
        extrasOptions[i]["amount"] +
        "<br> Is Paid: " +
        extrasOptions[i]["isPaid"] +
        "<br> Is Optional: " +
        extrasOptions[i]["isOptional"] +
        "<br><br>";
    }
    $("#modal-extras").modal("hide");
  }
}

function generateUUID() {
  // Public Domain/MIT
  var d = new Date().getTime(); //Timestamp
  var d2 = (performance && performance.now && performance.now() * 1000) || 0; //Time in microseconds since page-load or 0 if unsupported
  return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
    var r = Math.random() * 16; //random number between 0 and 16
    if (d > 0) {
      //Use timestamp until depleted
      r = (d + r) % 16 | 0;
      d = Math.floor(d / 16);
    } else {
      //Use microseconds since page-load if supported
      r = (d2 + r) % 16 | 0;
      d2 = Math.floor(d2 / 16);
    }
    return (c === "x" ? r : (r & 0x3) | 0x8).toString(16);
  });
}

function addMeal() {
  let mealName = document.getElementById("mealName").value;
  let mealPortion = document.getElementById("mealPortion").value;
  let mealCalories = document.getElementById("mealCalories").value;
  let mealPrice = document.getElementById("mealPrice").value;
  let mealDesc = document.getElementById("mealDesc").value;
  let mealPrepTime = document.getElementById("mealPrepTime").value;
  let mealNotes = document.getElementById("mealNotes").value;
  mealDiscount = document.getElementById("mealDiscount").value;
  let stCategoryIds = $("#mealCategory").select2("val");
  let stCategory = $("#mealCategory").select2("data");
  let mealRestriction = document.getElementById("mealQuantity").value;
  let categoryArray = [];
  for (let j = 0; j < stCategory.length; j++) {
    categoryArray.push(stCategory[j].text);
  }

  if ($("#chooseDrinks").select2("val").length >= 1) {
    drinksArrayIds = $("#chooseDrinks").select2("val");
    let stDrinks = $("#chooseDrinks").select2("data");
    for (let j = 0; j < stDrinks.length; j++) {
      drinksArray.push(stDrinks[j].text);
    }
  }
  if ($("#chooseCondiments").select2("val").length >= 1) {
    condimentsArrayIds = $("#chooseCondiments").select2("val");
    let stCondiments = $("#chooseCondiments").select2("data");
    for (let j = 0; j < stCondiments.length; j++) {
      condimentsArray.push(stCondiments[j].text);
    }
  }
  if ($("#chooseSide").select2("val").length >= 1) {
    sideMealIds = $("#chooseSide").select2("val");
    let stSide = $("#chooseSide").select2("data");
    for (let j = 0; j < stSide.length; j++) {
      sideMealArray.push(stSide[j].text);
    }
  }
  if ($("#chooseExtras").select2("val").length >= 1) {
    extrasArrayIds = $("#chooseExtras").select2("val");
    let stExtras = $("#chooseExtras").select2("data");
    for (let j = 0; j < stExtras.length; j++) {
      extrasArray.push(stExtras[j].text);
    }
  }

  if (mealName.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter name of the meal.",
    });
    return;
  }
  if (mealRestriction.length >= 1) {
    mealQuant = mealRestriction;
  }
  if (mealDesc.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter ingredients and description of the meal.",
    });
    return;
  }
  if (mealPrepTime.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter prepation time of the meal.",
    });
    return;
  }
  if (mealPortion.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter portion of the meal.",
    });
    return;
  }
  if (mealPrice.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter price of the meal.",
    });
    return;
  }

  if (categoryArray.length === 0) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please choose categories for you meal.",
    });
    return;
  }

  let statusCode;
  let state;
  if (document.getElementById("noNuts").checked) {
    statusCode = document.getElementById("noNuts").value;
    state = false;
  } else if (document.getElementById("yesNuts").checked) {
    statusCode = document.getElementById("yesNuts").value;
    state = true;
  }

  $("#stAddMeal")
    .html(
      '<span class="spinner-border spinner-border-sm mr-2" role="status" aria-hidden="true"></span>Loading...'
    )
    .attr("disabled", true);

  document.getElementById("stAddMeal").disabled = true;
  // Add a new document with a generated id.
  return db
    .collection("meals")
    .add({
      hasNuts: state,
      hasNutsCode: statusCode,
      restaurant: restName,
      restId: restId,
      mealName: mealName,
      mealPortion: mealPortion,
      restCategories: categoryArray,
      restCategoriesIds: stCategoryIds,
      mealPrice: mealPrice,
      mealDesc: mealDesc,
      mealDiscount: mealDiscount,
      mealPrepTime: mealPrepTime,
      mealCalories: mealCalories,
      mealNotes: mealNotes,
      mealDrinks: drinks,
      rating: 0,
      numberOfRatings: 0,
      quantityPerOrder: parseInt(mealQuant),
      drinksArray: drinksArray,
      drinksArrayIds: drinksArrayIds,
      sideMealArray: sideMealArray,
      sideMealIds: sideMealIds,
      condimentsArray: condimentsArray,
      condimentsArrayIds: condimentsArrayIds,
      extrasArray: extrasArray,
      extrasArrayIds: extrasArrayIds,
      mealImage: downloadUrl,
      createdByName: traderName,
      createdBy: uid,
      createdByEmail: email,
      status: "active",
      statusCode: 1,
      timestamp: firebase.firestore.FieldValue.serverTimestamp(),
    })
    .then(function (docRef) {
      console.log("Document written with ID: ", docRef.id);
      if (
        drinksOptions.length != 0 ||
        sideMealOptions.length != 0 ||
        condimentsOptions.length != 0 ||
        extrasOptions != 0
      ) {
        return db

          .collection("meals")
          .doc(docRef.id)
          .collection("extras")
          .add({
            drinks: drinksOptions,
            sideMeal: sideMealOptions,
            condiments: condimentsOptions,
            extras: extrasOptions,
          })
          .then(function () {
            Swal.fire({
              icon: "success",
              title: "Success",
              text: "You have successfully created a new meal.",
            }).then((result) => {
              window.location.href = "meal_list.html";
            });
          })
          .catch(function (err) {
            document.getElementById("stAddRest").disabled = false;
            $("#stAddMeal")
              .html(
                '<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n'
              )
              .attr("disabled", false);
            Swal.fire({
              icon: "error",
              title: "Oops...",
              text: err,
            }).then((result) => {});
          });
      } else {
        Swal.fire({
          icon: "success",
          title: "Success",
          text: "You have successfully created a new meal.",
        }).then((result) => {
          window.location.href = "meal_list.html";
        });
      }
    })
    .catch(function (error) {
      document.getElementById("stAddRest").disabled = false;
      $("#stAddMeal")
        .html(
          '<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n'
        )
        .attr("disabled", false);
      Swal.fire({
        icon: "error",
        title: "Oops...",
        text: error,
      }).then((result) => {});
    });
}

function isNumberKey(evt) {
  var charCode = evt.which ? evt.which : evt.keyCode;
  if (charCode > 31 && (charCode < 48 || charCode > 57)) return false;
  return true;
}
