/**
 * initApp handles setting up UI event listeners and registering Firebase auth listeners:
 *  - firebase.auth().onAuthStateChanged: This listener is called when the user is signed in or
 *    out, and that is where we update the UI.
 */

let email;
let uid;
let db = firebase.firestore();
let restName;
let restId;
let traderName;
let traderId;
let branchId;
let branchName;
let restPhoto = " ";
let username;
let statusCodeCollect = "0"
let stateCollect = false

var bs_modal = $('#modal');
var image = document.getElementById('image');
var cropper, reader, file;
let downloadUrl = ' ';

var maxwidth = 1024;
var maxheight = 500;

var finalAspectRatio = 1024 / 500;

var imgwidth = 0;
var imgheight = 0;



$("body").on("change", ".image", function (e) {
    var files = e.target.files;

    
    var done = function (url) {
        image.src = url;

        image.onload = function () {
            imgwidth = this.width;
            imgheight = this.height;
            if (imgwidth === maxwidth && imgheight === maxheight) {
                console.log("Image size is " + maxwidth + "X" + maxheight)
                bs_modal.modal('show');
            } else {

                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    text: 'Image size must be '+maxwidth+'w by '+maxheight+'h'

                })

            }
        }
        

    };


    if (files && files.length > 0) {
        file = files[0];

        if (URL) {
            done(URL.createObjectURL(file));
        } else if (FileReader) {
            reader = new FileReader();
            reader.onload = function (e) {
                done(reader.result);
            };
            reader.readAsDataURL(file);
        }
    }
});

bs_modal.on('shown.bs.modal', function () {
    $('#modal').modal({
        backdrop: 'static',
        keyboard: false
    })
    cropper = new Cropper(image, {
        aspectRatio: finalAspectRatio,
        viewMode: 1,
        preview: '.preview',

    });
}).on('hidden.bs.modal', function () {
    cropper.destroy();
    cropper = null;
});

$("#crop").click(function () {
    document.getElementById('crop').disabled = true;

    canvas = cropper.getCroppedCanvas({
        imageSmoothingEnabled: true,
    imageSmoothingQuality: 'high',
    });

    canvas.toBlob(function (blob) {
        url = URL.createObjectURL(blob);
        let imPreview = document.getElementById("imgPreview");



        var reader = new FileReader();
        reader.readAsDataURL(blob);
        reader.onloadend = function () {
            var base64data = reader.result;
            console.log(base64data);
            // Get a reference to the storage service, which is used to create references in your storage bucket
            const storage = firebase.storage();

            // Create a storage reference from our storage service
            const storageRef = storage.ref();
            storageRef.child('menus/' + generateUUID()).putString(base64data, 'data_url').then((snapshot) => {
                snapshot.ref.getDownloadURL().then(function (downloadURL) {
                    console.log('File available at', downloadURL);
                    downloadUrl = downloadURL;
                    imPreview.src = downloadURL;
                    document.getElementById('crop').disabled = false;

                    bs_modal.modal('hide');

                }).catch(function (error) {
                    document.getElementById('crop').disabled = false;

                    console.log("Error getting document:", error);
                });
                console.log('Uploaded a data_url string!');
            });



        };
    });
});

function initApp() {
    // Listening for auth state changes.
    // [START authstatelistener]
    firebase.auth().onAuthStateChanged(function (user) {
        if (user) {
            // User is signed in.

            email = user.email;
            let emailVerified = user.emailVerified;
            uid = user.uid;
            let providerData = user.providerData;

            // console.log(email)
            let db = firebase.firestore();


            let docRef = db.collection("branches").doc(uid);

            docRef.get().then(function (doc) {
                if (doc.exists) {

                    restName = doc.data().restaurant;
                    restId = doc.data().restId;
                    traderId = doc.data().branchId;
                    traderName = doc.data().managerFullName;
                    branchName = doc.data().branch;

                    document.getElementById('proName').innerText = traderName + " (" + branchName + ")";
                    var div = document.getElementById('chooseMeal');
                    db.collection("meals").where("restId", "==", restId)
                        .orderBy('timestamp', 'desc').onSnapshot((snapshot) => {
                            snapshot.forEach(function (docs) {
                                let mealId = docs.id;
                                let mealImg = docs.data().mealImage;
                                let mealName = docs.data().mealName;
                                let mealDesc = docs.data().mealDesc;
                                let mealPrice = docs.data().mealPrice;
                                let mealCategory = docs.data().restCategories;




                                div.innerHTML += '<option value=' + mealId + '>' + mealName + '</option>';


                            });


                        });



                }
            }).catch(function (error) {
                console.log("Error getting document:", error);
            });



        } else {
            // User is signed out.
            console.log("User is signed out");
            window.location.href = 'index.html'

        }

    });
    // [END authstatelistener]

}

window.onload = function () {
    initApp();
};



function toggleSignOut() {

    // [START signout]
    firebase.auth().signOut();
    // [END signout]

}

$(document).on("keydown", function (e) {

    var keyCode = e.which || e.keyCode;
    if (keyCode == 13) // enter key code
    {
        toggleSignIn();
    }

});
$(document).ready(function () {
    $("#mealCategory").select2();
    $("#chooseMeal").select2();

    $("#modal").modal({
        backdrop: "static",
        keyboard: false,
      });

});


function generateUUID() { // Public Domain/MIT
    var d = new Date().getTime();//Timestamp
    var d2 = (performance && performance.now && (performance.now() * 1000)) || 0;//Time in microseconds since page-load or 0 if unsupported
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = Math.random() * 16;//random number between 0 and 16
        if (d > 0) {//Use timestamp until depleted
            r = (d + r) % 16 | 0;
            d = Math.floor(d / 16);
        } else {//Use microseconds since page-load if supported
            r = (d2 + r) % 16 | 0;
            d2 = Math.floor(d2 / 16);
        }
        return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
}
function addMenu() {


    let menuName = document.getElementById('menuName').value;
    var checkBox = document.getElementById("checkAvailable");
    let startTime;
    let endTime;
    if (checkBox.checked == true) {
        startTime = document.getElementById('startTime').value;
        endTime = document.getElementById('endTime').value;
        if (startTime.length !== 5) {

            Swal.fire({
                icon: 'error',
                title: 'Oops...',
                text: 'Please enter a valid menu start time.'

            })
            return;
        }
        if (endTime.length !== 5) {

            Swal.fire({
                icon: 'error',
                title: 'Oops...',
                text: 'Please enter a valid menu end time.'

            })
            return;

        }
        startTime = startTime.replace(":", "");
        endTime = endTime.replace(":", "");

    } else {
        startTime = "0000";
        endTime = "2400";
    }



    let stCategoryIds = $('#mealCategory').select2("val");
    let stCategory = $('#mealCategory').select2("data");
    let categoryArray = []
    for (let j = 0; j < stCategory.length; j++) {
        categoryArray.push(stCategory[j].text)
    }

    let stMealIds = $('#chooseMeal').select2("val");
    let stMeals = $('#chooseMeal').select2("data");
    let mealArray = []
    for (let j = 0; j < stMeals.length; j++) {
        mealArray.push(stMeals[j].text)
    }
    if (document.getElementById('noCollect').checked) {
        statusCodeCollect = document.getElementById('noCollect').value;
        stateCollect = false;

    } else if (document.getElementById('yesCollect').checked) {
        statusCodeCollect = document.getElementById('yesCollect').value;
        stateCollect = true;
    }







    if (menuName.length < 1) {

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter name of the menu.'

        })
        return;
    }



    if (categoryArray.length === 0) {
        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please choose categories for you menu.'

        })
        return;
    }
    if (mealArray.length === 0) {
        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please choose meal for you menu.'

        })
        return;
    }




    $('#stAddMenu').html('<span class="spinner-border spinner-border-sm mr-2" role="status" aria-hidden="true"></span>Loading...').attr('disabled', true);

    document.getElementById('stAddMenu').disabled = true;

    // Add a new document with a generated id.
    return db.collection("menus").add({

        restaurant: restName,
        isCollectionOnly: stateCollect,
        isCollectionOnlyCode: statusCodeCollect,
        restId: restId,
        menuName: menuName,
        menuCategories: categoryArray,
        menuCategoriesIds: stCategoryIds,
        mealArray: mealArray,
        mealArrayIds: stMealIds,
        mealImage: downloadUrl,
        createdByName: traderName,
        createdBy: uid,
        rating: 0,
        numberOfRatings: 0,
        menuAvailability: checkBox.checked,
        startTime: parseInt(startTime),
        endTime: parseInt(endTime),
        createdByEmail: email,
        status: "active",
        statusCode: 1,
        timestamp: firebase.firestore.FieldValue.serverTimestamp(),
    })
        .then(function (docRef) {
            console.log("Document written " + docRef.id);
            for (let i = 0; i < stMealIds.length; i++) {
                console.log("Meal Id: " + stMealIds[i])
                db.collection("meals").doc(stMealIds[i]).update({
                    menuIds: firebase.firestore.FieldValue.arrayUnion(docRef.id)

                });
            }

            Swal.fire({
                icon: 'success',
                title: 'Success',
                text: 'You have successfully created a new menu.'

            }).then((result) => {
                window.location.href = 'menu_list.html'
            })
        })
        .catch(function (error) {
            document.getElementById('stAddMenu').disabled = false;
            $('#stAddMenu').html('<button type="button" class="btn btn-primary" id="stAddMenu" onclick="addMenu()">Save</button>\n').attr('disabled', false);
            Swal.fire({
                icon: 'error',
                title: 'Oops...',
                text: error

            }).then((result) => {

            })
        });





}
function openAvailability() {
    // Get the checkbox
    var checkBox = document.getElementById("checkAvailable");
    // Get the output text
    var availableDiv = document.getElementById("availableDiv");

    // If the checkbox is checked, display the output text
    if (checkBox.checked == true) {
        availableDiv.style.display = "block";
    } else {
        availableDiv.style.display = "none";
    }
}
function chunk(str, n) {
    var ret = [];
    var i;
    var len;

    for(i = 0, len = str.length; i < len; i += n) {
       ret.push(str.substr(i, n))
    }

    return ret
};