/**
 * initApp handles setting up UI event listeners and registering Firebase auth listeners:
 *  - firebase.auth().onAuthStateChanged: This listener is called when the user is signed in or
 *    out, and that is where we update the UI.
 */

let email;
let uid;
let placeSearch;
let autocomplete;
let lat;
let lng;
let restAddress;
let db = firebase.firestore();
let restId;
let downloadURL;
let traderName;

function initApp() {
    // Listening for auth state changes.
    // [START authstatelistener]
    firebase.auth().onAuthStateChanged(function (user) {
        if (user) {
            // User is signed in.



            email = user.email;
            let emailVerified = user.emailVerified;
            uid = user.uid;
            let providerData = user.providerData;

            console.log(user.metadata.lastSignInTime);
            let db = firebase.firestore();


            let docRef = db.collection("branches").doc(uid);

            docRef.get().then(function (doc) {
                if (doc.exists) {

                    let restName = doc.data().restaurant;
                     traderName = doc.data().traderFullName;
                    restId = doc.data().restId;
                    document.getElementById('proName').innerText = traderName + " (" + restName + ")";
                    let resRef = db.collection("restaurants").doc(doc.data().restId);
                    resRef.get().then(function (doc) {
                        if (doc.exists) {

                            document.getElementById('stName').value = doc.data().restaurant;
                            document.getElementById('stOpenTime').value = doc.data().openingTime;
                            document.getElementById('stCloseTime').value = doc.data().closingTime;


                            document.getElementById('stFloor').value = doc.data().restFloor;
                            // let status = document.getElementById('active').value;
                            document.getElementById('stFName').value = doc.data().managerFirstName;
                            document.getElementById('stLName').value = doc.data().managerLastName;

                            document.getElementById('stEmail').value = doc.data().restEmail;

                            document.getElementById('stPhone').value = doc.data().restPhone;
                            document.getElementById("storeAddress").value = doc.data().restAddress;
                            restAddress = doc.data().restAddress;
                            lat = doc.data().latitude;
                            lng = doc.data().longitude;

                            $('#restCat').val(null).trigger('change');
                            for (let i = 0; i < doc.data().restCategories.length; i++) {
                                var option = new Option(doc.data().restCategories[i], doc.data().restCategoriesIds[i], true, true);
                                $("#restCat").append(option).trigger('change');
                            }

                            $('#cuisineType').val(null).trigger('change');
                            for (let i = 0; i < doc.data().cuisineTypeArray.length; i++) {
                                var option = new Option(doc.data().cuisineTypeArray[i], doc.data().cuisineTypeIds[i], true, true);
                                $("#cuisineType").append(option).trigger('change');
                            }

                            if (doc.data().statusCode == "1") {
                                document.getElementById('radio1').checked = true;
                            } else {
                                document.getElementById('radio2').checked = true;
                            }
                            document.getElementById("imgPreview").src = doc.data().restImg;
                            downloadURL = doc.data().restImg;
                        }
                    }).catch(function (error) {
                        console.log("Error getting document:", error);
                    });




                } else {
                    Swal.fire({
                        icon: 'error',
                        title: 'Oops...',
                        text: 'You don not have access to this platform.'

                    });
                    toggleSignOut()
                    // doc.data() will be undefined in this case
                    console.log("No such document!");
                }
            }).catch(function (error) {
                console.log("Error getting document:", error);
            });



        } else {
            // User is signed out.
            console.log("User is signed out");
            window.location.href = 'index.html'

        }

    });
    // [END authstatelistener]

}

function initMap() {
    const center = { lat: 50.064192, lng: -130.605469 };
    // Create a bounding box with sides ~10km away from the center point
    const defaultBounds = {
        north: center.lat + 0.1,
        south: center.lat - 0.1,
        east: center.lng + 0.1,
        west: center.lng - 0.1,
    };
    const input = document.getElementById("storeAddress");
    const options = {
        bounds: defaultBounds,
        
        fields: ["geometry", "name", "formatted_address"],
        origin: center,
        strictBounds: false,
        types: ["establishment"],
    };
    autocomplete = new google.maps.places.Autocomplete(input, options);
    autocomplete.addListener("place_changed", fillInAddress);
}
window.onload = function () {
    initApp();
};

function fillInAddress() {
    // Get the place details from the autocomplete object.
    const place = autocomplete.getPlace();
    console.log(place);

    lat = place.geometry.location.lat();
    // get lng
    lng = place.geometry.location.lng();
    restAddress = place.name + ", " + place.formatted_address;

    console.log(lat + " " + lng + " " + restAddress);
}


function toggleSignOut() {

    // [START signout]
    firebase.auth().signOut();
    // [END signout]

}

$(document).on("keydown", function (e) {

    var keyCode = e.which || e.keyCode;
    if (keyCode == 13) // enter key code
    {
        toggleSignIn();
    }

});

$(document).ready(function () {
    $("#restCat").select2();
    $("#cuisineType").select2();
    let imFile = document.getElementById("findPhoto");
    let imPreview = document.getElementById("imgPreview");
    imFile.onchange = evt => {
        file = imFile.files[0]
        if (file) {
            imPreview.src = URL.createObjectURL(file)
        }
    }
});

let file = 'null';


function generateUUID() { // Public Domain/MIT
    var d = new Date().getTime();//Timestamp
    var d2 = (performance && performance.now && (performance.now() * 1000)) || 0;//Time in microseconds since page-load or 0 if unsupported
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = Math.random() * 16;//random number between 0 and 16
        if (d > 0) {//Use timestamp until depleted
            r = (d + r) % 16 | 0;
            d = Math.floor(d / 16);
        } else {//Use microseconds since page-load if supported
            r = (d2 + r) % 16 | 0;
            d2 = Math.floor(d2 / 16);
        }
        return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
}
let stBaking = " "
let stOils = " "
let stCondiments = " "
let stNuts = " "
let stDelCond = " "
let stKnockDown = " "

function addRest() {
    $('#stAddRest').html('<span class="spinner-border spinner-border-sm mr-2" role="status" aria-hidden="true"></span>Loading...').attr('disabled', true);

    document.getElementById('stAddRest').disabled = true;


    let stCategoryIds = $('#restCat').select2("val");
    let stCategory = $('#restCat').select2("data");
    let categoryArray = []
    for (let j = 0; j < stCategory.length; j++) {
        categoryArray.push(stCategory[j].text)
    }

    let cuisineTypeIds = $('#cuisineType').select2("val");
    let cuisineType = $('#cuisineType').select2("data");
    let cuisineTypeArray = []
    for (let j = 0; j < cuisineType.length; j++) {
        cuisineTypeArray.push(cuisineType[j].text)
    }

    let stName = document.getElementById('stName').value;
    let stOpenTime = document.getElementById('stOpenTime').value;
    let stCloseTime = document.getElementById('stCloseTime').value;


    let stFloor = document.getElementById('stFloor').value;
    // let status = document.getElementById('active').value;
    let stFName = document.getElementById('stFName').value;
    let stLName = document.getElementById('stLName').value;

    let stEmail = document.getElementById('stEmail').value;

    let stPhone = document.getElementById('stPhone').value;

    // stBaking = document.getElementById('stBaking').value;
    // stOils = document.getElementById('stOils').value;
    //  stNuts = document.getElementById('stNuts').value;
    // stCondiments = document.getElementById('stCondiments').value;
    // stDelCond = document.getElementById('stDelCond').value;
    // stKnockDown = document.getElementById('stKnockDown').value;

    let stImage = document.getElementById('findPhoto');

    // Get a reference to the storage service, which is used to create references in your storage bucket
    const storage = firebase.storage();

    // Create a storage reference from our storage service
    const storageRef = storage.ref();


    

    if (stName.length < 1) {
        document.getElementById('stAddRest').disabled = false;
        $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a store name.'

        })
        return;
    }
    if (stFloor.length < 1) {
        document.getElementById('stAddRest').disabled = false;
        $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a floor/suite.'

        })
        return;
    }
    if (stFName.length < 1) {
        document.getElementById('stAddRest').disabled = false;
        $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a first name.'

        })
        return;
    }
    if (stLName.length < 1) {
        document.getElementById('stAddRest').disabled = false;
        $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please choose a last name.'

        })
        return;
    }
    console.log("lat", lat);
    console.log("restAddress", restAddress);
    if (lat === null || typeof restAddress === "undefined") {
        document.getElementById('stAddRest').disabled = false;
        $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please choose a store address.'

        })
        return;
    }
    if (stPhone === null) {
        document.getElementById('stAddRest').disabled = false;
        $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter phone number'

        });
        return;
    }
    if (stEmail === null) {
        document.getElementById('stAddRest').disabled = false;
        $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter physical address'

        });
        return;
    }

    if (categoryArray.length === 0) {
        document.getElementById('stAddRest').disabled = false;
        $('#addStore').html('<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter restaurant category.'

        })
        return;
    }
    if (cuisineTypeArray.length === 0) {
        document.getElementById('stAddRest').disabled = false;
        $('#addStore').html('<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter cuisine type.'

        })

        return;
    }
    if (stOpenTime === null) {
        document.getElementById('stAddRest').disabled = false;
        $('#addStore').html('<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter opening time.'

        })

        return;
    }
    if (stCloseTime.length === null) {
        document.getElementById('stAddRest').disabled = false;
        $('#addStore').html('<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter closing time.'

        })

        return;
    }

    if (document.getElementById('radio1').checked === false || !document.getElementById('radio2').checked === false) {
        document.getElementById('stAddRest').disabled = false;
        $('#addStore').html('<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please choose a status.'

        })

        return;
    }

    let statusCode;
    let state;
    if (document.getElementById('radio1').checked) {
        statusCode = document.getElementById('radio1').value;
        state = "Published";

    } else if (document.getElementById('radio2').checked) {
        statusCode = document.getElementById('radio2').value;
        state = "Draft";
    }
    if (typeof file === "undefined" || file === "null") {
         // Add a new document with a generated id.
         return db.collection("restaurants").doc(restId).update({
            status: state,
            restId: restId,
            statusCode: statusCode,
            restaurant: stName,
            restAddress: restAddress,
            restFloor: stFloor,
            managerFirstName: stFName,
            restCategories: categoryArray,
            restCategoriesIds: stCategoryIds,
            cuisineTypeIds: cuisineTypeIds,
            cuisineTypeArray: cuisineTypeArray,
            managerLastName: stLName,
            managerFullName: stFName + " " + stLName,
            restEmail: stEmail,
            restPhone: stPhone,
            latitude: lat,
            longitude: lng,
            openingTime: stOpenTime,
            closingTime: stCloseTime,
            updatedByName: traderName,
            updatedBy: uid,
            updatedByEmail: email,
            restImg: downloadURL,
            updatedOn: firebase.firestore.FieldValue.serverTimestamp(),
        })
            .then(function () {
                console.log("Document written ");
                Swal.fire({
                    icon: 'success',
                    title: 'Success',
                    text: 'You have successfully updated restaurant.'

                }).then((result) => {
                    location.reload(); 
                })


            })
            .catch(function (error) {
                $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
                console.error("Error adding document: ", error);
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    text: error

                }).then((result) => {

                })
            });

    } else {
        let uploadTask = storageRef.child('restaurants/' + generateUUID()).put(file);
        // Listen for state changes, errors, and completion of the upload.
        return uploadTask.on(firebase.storage.TaskEvent.STATE_CHANGED, // or 'state_changed'
            function (snapshot) {
                // Get task progress, including the number of bytes uploaded and the total number of bytes to be uploaded
                var progress = (snapshot.bytesTransferred / snapshot.totalBytes) * 100;
                console.log('Upload is ' + progress + '% done');
                switch (snapshot.state) {
                    case firebase.storage.TaskState.PAUSED: // or 'paused'
                        console.log('Upload is paused');
                        break;
                    case firebase.storage.TaskState.RUNNING: // or 'running'
                        console.log('Upload is running');
                        break;
                }
            }, function (error) {
                document.getElementById('stAddRest').disabled = false;
                $('#addStore').html('<button type="stAddRest" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);

                // A full list of error codes is available at
                // https://firebase.google.com/docs/storage/web/handle-errors
                switch (error.code) {
                    case 'storage/unauthorized':
                        // User doesn't have permission to access the object
                        Swal.fire({
                            icon: 'error',
                            title: 'Oops...',
                            text: error

                        });
                        break;

                    case 'storage/canceled':
                        // User canceled the upload
                        Swal.fire({
                            icon: 'error',
                            title: 'Oops...',
                            text: error

                        });
                        break;


                    case 'storage/unknown':
                        // Unknown error occurred, inspect error.serverResponse
                        Swal.fire({
                            icon: 'error',
                            title: 'Oops...',
                            text: error

                        });
                        break;

                }
            }, function () {
                // Upload completed successfully, now we can get the download URL
                uploadTask.snapshot.ref.getDownloadURL().then(function (downloadURL) {



                    // Add a new document with a generated id.
                    return db.collection("restaurants").doc(restId).update({
                        status: state,
                        restId: restId,
                        statusCode: statusCode,
                        restaurant: stName,
                        restAddress: restAddress,
                        restFloor: stFloor,
                        managerFirstName: stFName,
                        restCategories: categoryArray,
                        restCategoriesIds: stCategoryIds,
                        cuisineTypeIds: cuisineTypeIds,
                        cuisineTypeArray: cuisineTypeArray,
                        managerLastName: stLName,
                        managerFullName: stFName + " " + stLName,
                        restEmail: stEmail,
                        restPhone: stPhone,
                        latitude: lat,
                        longitude: lng,
                        openingTime: stOpenTime,
                        closingTime: stCloseTime,
                        updatedByName: username,
                        updatedBy: uid,
                        updatedByEmail: email,
                        restImg: downloadURL,
                        updatedOn: firebase.firestore.FieldValue.serverTimestamp(),
                    })
                        .then(function () {
                            console.log("Document written ");
                            Swal.fire({
                                icon: 'success',
                                title: 'Success',
                                text: 'You have successfully updated restaurant.'

                            }).then((result) => {
                                location.reload(); 
                            })


                        })
                        .catch(function (error) {
                            $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
                            console.error("Error adding document: ", error);
                            Swal.fire({
                                icon: 'error',
                                title: 'Oops...',
                                text: error

                            }).then((result) => {

                            })
                        });
                });
            }
        );
    }



}